import React, { useRef, useState, useEffect } from 'react';
import { Terminal } from 'lucide-react';
import TerminalButton from './components/TerminalButton';
import ProfileContent from './components/ProfileContent';
import ProjectsContent from './components/ProjectsContent';
import TerminalShell from './components/TerminalShell';
import SkillsContent from './components/SkillsContent';
import ResumeContent from './components/ResumeContent';
import PostsContent from './components/PostsContent';

import './styles/sliders.css'; // Import the new sliders CSS

type SliderRefType = React.RefObject<HTMLDivElement>;

function App() {
  const [activeSection, setActiveSection] = React.useState('about');
  const [isBooting, setIsBooting] = React.useState(true);
  const [bootPhase, setBootPhase] = React.useState(0);
  
  // New state for sliders
  const [greenIntensity, setGreenIntensity] = useState(50); // 0 to 100
  const [glitchIntensity, setGlitchIntensity] = useState(25); // 0 to 100
  
  // New state for glitch effects
  const [flickerOpacity, setFlickerOpacity] = useState(1); // 1 is normal, less than 1 creates flicker
  const [showScanline, setShowScanline] = useState(false); // Controls the high-contrast scanline
  const [scanlinePosition, setScanlinePosition] = useState(0); // Position of scanline (0-100%)
  
  // Refs for the slider thumbs
  const greenSliderRef = useRef<HTMLDivElement>(null);
  const glitchSliderRef = useRef<HTMLDivElement>(null);

  // TV Boot-up sequence
  React.useEffect(() => {
    if (!isBooting) return;
    // Phase 0: Initial black screen (0ms)
    // Phase 1: Quick white flash (100ms)
    // Phase 2: Static noise (600ms)
    // Phase 3: Horizontal scan line (900ms) 
    // Phase 4: Fade in content (1200ms)
    // Phase 5: Boot complete (1800ms)
    
    const timings = [0, 100, 600, 900, 1200, 1800];
    
    const bootSequence = timings.map((time, index) => {
      return setTimeout(() => {
        setBootPhase(index);
        if (index === timings.length - 1) {
          setIsBooting(false);
        }
      }, time);
    });
    
    return () => {
      bootSequence.forEach(timeout => clearTimeout(timeout));
    };
  }, [isBooting]);

  // Screen flickering effect based on glitch intensity
  useEffect(() => {
    if (glitchIntensity <= 30) {
      setFlickerOpacity(1); // No flickering
      return;
    }
    
    let flickerInterval: number;
    const flickerEffect = () => {
      // The higher the glitch intensity, the more frequent the flickers
      const flickerFrequency = Math.max(300, 1500 - (glitchIntensity * 10));
      
      flickerInterval = window.setInterval(() => {
        // Intensity affects how dim the screen gets during flickers
        const minOpacity = Math.max(0.7, 0.95 - (glitchIntensity / 100) * 0.5);
        
        // Create random flickering effect
        const randomFlicker = Math.random() * (1 - minOpacity) + minOpacity;
        setFlickerOpacity(randomFlicker);
        
        // Reset back to normal after a short delay
        setTimeout(() => {
          setFlickerOpacity(1);
        }, 50 + Math.random() * 100);
      }, flickerFrequency);
    };
    
    flickerEffect();
    
    return () => {
      clearInterval(flickerInterval);
    };
  }, [glitchIntensity]);
  
  // Horizontal scanline effect for higher glitch intensity
  useEffect(() => {
    if (glitchIntensity <= 50) {
      setShowScanline(false);
      return;
    }
    
    let scanlineInterval: number;
    const scanlineEffect = () => {
      // The higher the intensity, the more frequent the scanline appears
      const scanlineFrequency = Math.max(1000, 5000 - (glitchIntensity * 40));
      
      scanlineInterval = window.setInterval(() => {
        // Show the scanline
        setShowScanline(true);
        
        // Animate the scanline movement
        const duration = 300 + Math.random() * 700; // 300-1000ms
        const startTime = Date.now();
        
        const animateScanline = () => {
          const elapsed = Date.now() - startTime;
          const progress = Math.min(1, elapsed / duration);
          
          setScanlinePosition(progress * 100);
          
          if (progress < 1) {
            requestAnimationFrame(animateScanline);
          } else {
            // Hide the scanline when animation completes
            setShowScanline(false);
          }
        };
        
        requestAnimationFrame(animateScanline);
      }, scanlineFrequency);
    };
    
    scanlineEffect();
    
    return () => {
      clearInterval(scanlineInterval);
    };
  }, [glitchIntensity]);

  // Function to handle slider dragging
  const handleSliderDrag = (
    event: React.MouseEvent, 
    sliderRef: SliderRefType,
    setValueFn: React.Dispatch<React.SetStateAction<number>>
  ) => {
    event.preventDefault();
    
    if (!sliderRef.current) return;
    
    const track = sliderRef.current.parentElement;
    if (!track) return;
    
    const trackRect = track.getBoundingClientRect();
    const trackHeight = trackRect.height;
    const trackTop = trackRect.top;
    
    const handleMouseMove = (moveEvent: MouseEvent) => {
      const y = moveEvent.clientY;
      let percentage = 100 - (((y - trackTop) / trackHeight) * 100);
      
      // Clamp value between 0 and 100
      percentage = Math.min(100, Math.max(0, percentage));
      
      setValueFn(percentage);
    };
    
    const handleMouseUp = () => {
      document.removeEventListener('mousemove', handleMouseMove);
      document.removeEventListener('mouseup', handleMouseUp);
    };
    
    document.addEventListener('mousemove', handleMouseMove);
    document.addEventListener('mouseup', handleMouseUp);
  };

  // Calculate green color based on intensity
  const calculateGreenBackground = () => {
    // Map the intensity from 0-100 to appropriate color values
    const baseIntensity = Math.floor((greenIntensity / 100) * 35); // Max 35
    return {
      backgroundColor: `rgba(0, ${baseIntensity}, 0, 0.95)`,
      backgroundImage: `
        linear-gradient(0deg, 
          rgba(0, ${Math.max(5, baseIntensity + 3)}, 0, 1) 0%, 
          rgba(0, ${Math.max(15, baseIntensity + 10)}, 0, 1) 100%),
        radial-gradient(
          circle at center, 
          rgba(0, ${Math.max(20, baseIntensity + 20)}, 0, 1) 0%, 
          rgba(0, ${baseIntensity}, 0, 1) 100%
        )
      `
    };
  };

  // Calculate glitch effects based on intensity
  const calculateGlitchEffects = () => {
    const whiteNoiseOpacity = 0.02 + (glitchIntensity / 100) * 0.2;
    const chromaticAberrationAmount = (glitchIntensity / 100) * 1;
    const scanLinesOpacity = 0.1 + (glitchIntensity / 100) * 1;
    
    return {
      whiteNoiseOpacity,
      chromaticAberrationAmount,
      scanLinesOpacity
    };
  };
  
  const glitchEffects = calculateGlitchEffects();

  return (
    <div className="min-h-screen bg-gray-900 flex items-center justify-center overflow-hidden">
      {/* TV Container with fixed proportions */}
      <div className="w-[95%] max-w-[1400px] relative h-[90vh]">
        {/* Vertical Sliders - Moved outside the TV frame */}
        <div className="tv-sliders-container">
          {/* Green Intensity Slider */}
          <div className="tv-slider-wrapper">
            <div className="tv-slider-track">
              <div 
                ref={greenSliderRef}
                className="tv-slider-thumb" 
                style={{ bottom: `${greenIntensity}%` }}
                onMouseDown={(e) => handleSliderDrag(e, greenSliderRef, setGreenIntensity)}
              ></div>
            </div>
            <span className="tv-slider-label">Green</span>
          </div>
          
          {/* Glitch Intensity Slider */}
          <div className="tv-slider-wrapper">
            <div className="tv-slider-track">
              <div 
                ref={glitchSliderRef}
                className="tv-slider-thumb" 
                style={{ bottom: `${glitchIntensity}%` }}
                onMouseDown={(e) => handleSliderDrag(e, glitchSliderRef, setGlitchIntensity)}
              ></div>
            </div>
            <span className="tv-slider-label">Glitch</span>
          </div>
        </div>

        {/* TV Frame */}
        <div className="absolute inset-0 tv-frame">
          {/* TV frame texture overlay */}
          <div className="tv-frame-texture"></div>
          
          {/* Edge shadow for added depth */}
          <div className="tv-frame-edge"></div>
          
          {/* Screen Container with fixed height and Convex Effect */}
          <div className="relative w-full h-full rounded-[20px] overflow-hidden tv-screen-container">
            {/* Convex Screen Effect */}
            <div className="absolute inset-0 tv-screen-convex"></div>
            
            {/* Boot-up Animation Layers */}
            {isBooting && (
              <>
                {/* Black Screen (initial) */}
                <div className={`absolute inset-0 bg-black z-30 transition-opacity duration-500 ${bootPhase >= 1 ? 'opacity-0' : 'opacity-100'}`}></div>
                
                {/* Power On Flash */}
                <div className={`absolute inset-0 bg-white z-30 transition-opacity duration-300 ${bootPhase === 1 ? 'opacity-80' : 'opacity-0'}`}></div>
                
                {/* Intense Static */}
                <div className={`absolute inset-0 z-30 transition-opacity duration-300 ${bootPhase === 2 ? 'opacity-100' : 'opacity-0'}`}>
                  <div className="absolute inset-0 bg-black opacity-30"></div>
                  <div className="absolute inset-0" style={{
                    backgroundImage: `url("data:image/svg+xml,%3Csvg viewBox='0 0 250 250' xmlns='http://www.w3.org/2000/svg'%3E%3Cfilter id='noiseFilter'%3E%3CfeTurbulence type='fractalNoise' baseFrequency='0.65' numOctaves='3' stitchTiles='stitch'/%3E%3C/filter%3E%3Crect width='100%25' height='100%25' filter='url(%23noiseFilter)'/%3E%3C/svg%3E")`,
                    opacity: 1
                  }}></div>
                </div>
                
                {/* Horizontal Scan Line */}
                {bootPhase === 3 && (
                  <div className="absolute inset-0 z-30">
                    <div className="h-8 w-full bg-white opacity-60 animate-scanline"></div>
                  </div>
                )}
                
                {/* Content Fade Overlay */}
                <div className={`absolute inset-0 bg-black z-20 pointer-events-none transition-opacity duration-500 ${bootPhase >= 4 ? 'opacity-0' : 'opacity-100'}`}></div>
              </>
            )}
            
            {/* Static noise texture underlay - controlled by glitch slider */}
            <div 
              className="white-noise" 
              style={{ opacity: glitchEffects.whiteNoiseOpacity }}
            ></div>
            
            {/* Screen opacity flicker effect */}
            <div 
              className="absolute inset-0 z-45 pointer-events-none transition-opacity duration-50"
              style={{ 
                opacity: 1 - flickerOpacity, // Inverse of flickerOpacity to create dimming
                backgroundColor: 'rgba(0, 0, 0, 0.95)' // Black overlay for flickering
              }}
            ></div>
            
            {/* High-contrast horizontal scanline for high glitch intensity */}
            {showScanline && (
              <div 
                className="absolute left-0 right-0 z-45 pointer-events-none"
                style={{ 
                  top: `${scanlinePosition}%`, 
                  height: '4px',
                  backgroundColor: 'rgba(255, 255, 255, 0.7)',
                  boxShadow: '0 0 10px rgba(255, 255, 255, 0.7), 0 0 5px rgba(255, 255, 255, 0.5)',
                  borderTop: '1px solid rgba(255, 255, 255, 0.9)',
                  borderBottom: '1px solid rgba(255, 255, 255, 0.9)'
                }}
              ></div>
            )}
            
            {/* Screen Content with fixed height - controlled by green slider */}
            <div 
              className="relative h-full w-full p-6 font-mono text-[#00FF00] overflow-hidden z-10 screen-content"
              style={{
                ...calculateGreenBackground(),
                opacity: flickerOpacity // Apply flickering effect
              }}
            >
              <div className="animate-pulse mb-4">
                <Terminal className="inline-block mr-2" />
                <span className="text-sm">echo "Hey there!"</span>
              </div>
              {/* Split Screen Container with fixed height */}
              <div className="flex gap-8 h-[calc(100%-3rem)]">
                {/* Left Column - Navigation */}
                <div className="w-64 space-y-3 overflow-y-auto custom-scrollbar">
                  <TerminalButton
                    onClick={() => setActiveSection('about')}
                    isSelected={activeSection === 'about'}
                  >
                    ABOUT
                  </TerminalButton>
                  <TerminalButton
                    onClick={() => setActiveSection('projects')}
                    isSelected={activeSection === 'projects'}
                  >
                    PROJECTS
                  </TerminalButton>
                  <TerminalButton
                    onClick={() => setActiveSection('skills')}
                    isSelected={activeSection === 'skills'}
                  >
                    SKILLS
                  </TerminalButton>
                  <TerminalButton
                    onClick={() => setActiveSection('posts')}
                    isSelected={activeSection === 'posts'}
                  >
                    POSTS
                  </TerminalButton>
                  <TerminalButton
                    onClick={() => setActiveSection('resume')}
                    isSelected={activeSection === 'resume'}
                  >
                    RESUME
                  </TerminalButton>
                  <TerminalButton
                    onClick={() => setActiveSection('shell')}
                    isSelected={activeSection === 'shell'}
                  >
                    SHELL
                  </TerminalButton>
                </div>
                {/* Vertical Separator with embossed effect */}
                <div className="vertical-separator"></div>
                {/* Right Column - Content with scrolling */}
                <div className="flex-1 overflow-y-auto pr-4 custom-scrollbar">
                  {activeSection === 'about' && <ProfileContent />}
                  {activeSection === 'projects' && (
                    <ProjectsContent markdownPath="/content/projects.md" />
                  )}
                  {activeSection === 'skills' && <SkillsContent />}
                  {activeSection === 'posts' && <PostsContent />}
                  {activeSection === 'resume' && <ResumeContent />}
                  {activeSection === 'shell' && <TerminalShell />}
                </div>
              </div>
            </div>
            {/* Enhanced Scan Lines Effect - controlled by glitch slider */}
            <div 
              className="scan-lines"
              style={{
                backgroundImage: `repeating-linear-gradient(
                  to bottom,
                  rgba(0, 0, 0, 0) 0px,
                  rgba(0, 0, 0, 0) 1px,
                  rgba(0, 0, 0, ${glitchEffects.scanLinesOpacity}) 1px,
                  rgba(0, 0, 0, ${glitchEffects.scanLinesOpacity}) 2px
                )`
              }}
            ></div>
            
            {/* Chromatic Aberration Effect - controlled by glitch slider */}
            <div 
              className="chromatic-aberration"
              style={{
                boxShadow: `
                  inset ${glitchEffects.chromaticAberrationAmount}px 0 0 rgba(255, 0, 0, 0.05),
                  inset -${glitchEffects.chromaticAberrationAmount}px 0 0 rgba(0, 255, 255, 0.05)
                `
              }}
            ></div>
            
            {/* Multiple Glass Reflection Layers */}
            <div className="glass-reflection"></div>
            <div className="glass-reflection-2"></div>
            <div className="corner-shine"></div>
            {/* Screen Glare - Moving reflection */}
            <div className="screen-glare"></div>
          </div>
        </div>
        {/* TV Controls - Skeuomorphic Knobs */}
        <div className="absolute -bottom-12 right-20 flex space-x-8">
          <div className="tv-knob" style={{transform: 'rotate(-30deg)'}}></div>
          <div className="tv-knob" style={{transform: 'rotate(15deg)'}}></div>
          <div className="tv-knob" style={{transform: 'rotate(-10deg)'}}></div>
        </div>
        {/* TV Speaker Grills - Enhanced */}
        <div className="absolute -bottom-6 left-20 flex space-x-1.5">
          {[...Array(10)].map((_, i) => (
            <div 
              key={i} 
              className="w-1.5 h-6 bg-gradient-to-b from-gray-600 to-gray-800 rounded-full"
              style={{
                boxShadow: 'inset 0 1px 1px rgba(255,255,255,0.1), inset 0 -1px 1px rgba(0,0,0,0.3)'
              }}
            ></div>
          ))}
        </div>
        {/* TV Brand Label */}
        <div 
          className="absolute -bottom-10 left-1/2 transform -translate-x-1/2 bg-gradient-to-b from-gray-700 to-gray-900 px-4 py-1 rounded"
          style={{
            boxShadow: 'inset 0 1px 1px rgba(255,255,255,0.2), 0 1px 2px rgba(0,0,0,0.5)',
            border: '1px solid rgba(0,0,0,0.3)'
          }}
        >
          <span 
            className="text-xs text-gray-300 font-bold"
            style={{
              textShadow: '0 -1px 0 rgba(0,0,0,0.5)'
            }}
          >
            RETRO·VISION
          </span>
        </div>
        {/* Power indicator light */}
        <div className="absolute -bottom-8 right-[12%] w-3 h-3 rounded-full bg-gradient-to-b from-green-500 to-green-700" 
          style={{
            boxShadow: '0 0 5px #4AFF4A, inset 0 1px 2px rgba(255,255,255,0.4), inset 0 -1px 2px rgba(0,0,0,0.4)'
          }}>
          <div className="absolute inset-0 rounded-full bg-green-400 opacity-50 animate-pulse"></div>
        </div>
      </div>
    </div>
  );
}

export default App;